import sys
import trio
from abc import abstractmethod, abstractproperty, ABCMeta
from typing import (
    Any,
    AsyncIterator,
    Awaitable,
    Callable,
    FrozenSet,
    Generic,
    Optional,
    Type,
    TypeVar,
    Union,
    overload,
)
from types import CodeType, FrameType, TracebackType
from typing_extensions import Protocol
from mypy_extensions import NamedArg, VarArg

__all__ = [
    "Nursery",
    "TaskStatus",
    "takes_callable_and_args",
    "AsyncGenerator",
    "CompatAsyncGenerator",
]

# backward compatibility
from trio import Nursery as Nursery

T = TypeVar("T")
T_co = TypeVar("T_co", covariant=True)
T_co2 = TypeVar("T_co2", covariant=True)
T_contra = TypeVar("T_contra", contravariant=True)

def takes_callable_and_args(fn: T) -> T:
    return fn

class TaskStatus(Protocol[T_contra]):
    def started(self, value: T_contra = ...) -> None: ...

if sys.version_info >= (3, 6):
    from typing import AsyncGenerator as AsyncGenerator
else:
    class AsyncGenerator(AsyncIterator[T_co], Generic[T_co, T_contra]):
        @abstractmethod
        async def __anext__(self) -> T_co: ...
        @abstractmethod
        async def asend(self, value: T_contra) -> T_co: ...
        @abstractmethod
        async def athrow(
            self,
            exc_type: Type[BaseException],
            exc_value: Optional[BaseException] = ...,
            exc_traceback: Optional[TracebackType] = ...,
        ) -> T_co: ...
        @abstractmethod
        async def aclose(self) -> None: ...
        @abstractmethod
        def __aiter__(self) -> AsyncGenerator[T_co, T_contra]: ...
        @property
        def ag_await(self) -> Any: ...
        @property
        def ag_code(self) -> CodeType: ...
        @property
        def ag_frame(self) -> FrameType: ...
        @property
        def ag_running(self) -> bool: ...

class CompatAsyncGenerator(
    AsyncGenerator[T_co, T_contra], Generic[T_co, T_contra, T_co2], metaclass=ABCMeta
):
    async def __anext__(self) -> T_co: ...
    async def asend(self, value: T_contra) -> T_co: ...
    async def athrow(
        self,
        exc_type: Type[BaseException],
        exc_value: Optional[BaseException] = ...,
        exc_traceback: Optional[TracebackType] = ...,
    ) -> T_co: ...
    # aclose() should return None but the stubs in typeshed for 3.6+ say
    # it returns the YieldType, so we need to use Any to be compatible
    # (https://github.com/python/typeshed/issues/2785)
    async def aclose(self) -> Any: ...
    def __aiter__(self) -> AsyncGenerator[T_co, T_contra]: ...
    @property
    def ag_await(self) -> Any: ...
    @property
    def ag_code(self) -> CodeType: ...
    @property
    def ag_frame(self) -> FrameType: ...
    @property
    def ag_running(self) -> bool: ...

class YieldType(Generic[T_co]):
    pass

class SendType(Generic[T_contra]):
    pass



from typing import (
    Any,
    AsyncContextManager,
    AsyncIterator,
    Awaitable,
    Callable,
    ContextManager,
    Coroutine,
    Generic,
    NoReturn,
    Optional,
    Sequence,
    Union,
    Sequence,
    TypeVar,
    Tuple,
)
from trio_typing import Nursery, takes_callable_and_args
from mypy_extensions import VarArg
import trio
import outcome
import contextvars
import enum
import select

T = TypeVar("T")
F = TypeVar("F", bound=Callable[..., Any])

class _Statistics:
    def __getattr__(self, name: str) -> Any: ...

# _core._ki
def enable_ki_protection(fn: F) -> F: ...
def disable_ki_protection(fn: F) -> F: ...
def currently_ki_protected() -> bool: ...

# _core._entry_queue
class TrioToken:
    @takes_callable_and_args
    def run_sync_soon(
        self,
        sync_fn: Union[Callable[..., None], Callable[[VarArg()], None]],
        *args: Any,
        idempotent: bool = False,
    ) -> None: ...

# _core._unbounded_queue
class UnboundedQueue(Generic[T]):
    def __init__(self) -> None: ...
    def qsize(self) -> int: ...
    def empty(self) -> bool: ...
    def put_nowait(self, obj: T) -> None: ...
    def get_batch_nowait(self) -> Sequence[T]: ...
    async def get_batch(self) -> Sequence[T]: ...
    def statistics(self) -> _Statistics: ...
    def __aiter__(self) -> AsyncIterator[Sequence[T]]: ...

# _core._run
class Task:
    coro: Coroutine[Any, outcome.Outcome[object], Any]
    name: str
    context: contextvars.Context
    custom_sleep_data: Any
    parent_nursery: Optional[Nursery]
    child_nurseries: Sequence[Nursery]

async def checkpoint() -> None: ...
async def checkpoint_if_cancelled() -> None: ...
def current_task() -> Task: ...
def current_root_task() -> Task: ...
def current_statistics() -> _Statistics: ...
def current_clock() -> trio.abc.Clock: ...
def current_trio_token() -> TrioToken: ...
def reschedule(task: Task, next_send: outcome.Outcome[Any] = ...) -> None: ...
@takes_callable_and_args
def spawn_system_task(
    async_fn: Union[
        Callable[..., Awaitable[None]], Callable[[VarArg()], Awaitable[None]]
    ],
    *args: Any,
    name: object = ...,
) -> Task: ...
def add_instrument(instrument: trio.abc.Instrument) -> None: ...
def remove_instrument(instrument: trio.abc.Instrument) -> None: ...
async def wait_readable(fd: int) -> None: ...
async def wait_writable(fd: int) -> None: ...
def notify_closing(fd: int) -> None: ...

# kqueue only
def current_kqueue() -> select.kqueue: ...
def monitor_kevent(
    ident: int, filter: int
) -> ContextManager[UnboundedQueue[select.kevent]]: ...
async def wait_kevent(
    ident: int, filter: int, abort_func: Callable[[Callable[[], NoReturn]], Abort]
) -> select.kevent: ...

# windows only
class _CompletionKeyEventInfo:
    lpOverlapped: int
    dwNumberOfBytesTransferred: int

def current_iocp() -> int: ...
def register_with_iocp(handle: int) -> None: ...
async def wait_overlapped(handle: int, lpOverlapped: int) -> None: ...
def monitor_completion_key() -> ContextManager[
    Tuple[int, UnboundedQueue[_CompletionKeyEventInfo]]
]: ...

# _core._traps
class Abort(enum.Enum):
    SUCCEEDED = ...
    FAILED = ...

async def cancel_shielded_checkpoint() -> None: ...
async def wait_task_rescheduled(
    abort_func: Callable[[Callable[[], NoReturn]], Abort]
) -> Any: ...
async def permanently_detach_coroutine_object(
    final_outcome: outcome.Outcome[object]
) -> Any: ...
async def temporarily_detach_coroutine_object(
    abort_func: Callable[[Callable[[], NoReturn]], Abort]
) -> Any: ...
async def reattach_detached_coroutine_object(task: Task, yield_value: Any) -> None: ...

# _core._parking_lot
class ParkingLot:
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    async def park(self) -> None: ...
    def unpark(self, *, count: int = 1) -> Sequence[Task]: ...
    def unpark_all(self) -> Sequence[Task]: ...
    def repark(self, new_lot: ParkingLot, *, count: int = 1) -> None: ...
    def repark_all(self, new_lot: ParkingLot) -> None: ...
    def statistics(self) -> _Statistics: ...

# _core._local
class _RunVarToken:
    pass

class RunVar(Generic[T]):
    def __init__(self, name: str, default: T = ...) -> None: ...
    def get(self, default: T = ...) -> T: ...
    def set(self, value: T) -> _RunVarToken: ...
    def reset(self, token: _RunVarToken) -> None: ...

# _unix_pipes
class FdStream(trio.abc.Stream):
    def __init__(self, fd: int): ...
    async def send_all(self, data: Union[bytes, memoryview]) -> None: ...
    async def wait_send_all_might_not_block(self) -> None: ...
    async def receive_some(self, max_bytes: Optional[int] = ...) -> bytes: ...
    async def aclose(self) -> None: ...
    def fileno(self) -> int: ...

# _wait_for_object
async def WaitForSingleObject(obj: int) -> None: ...



import trio
from abc import ABCMeta, abstractmethod
from typing import List, Tuple, Union, Any, Optional, Generic, TypeVar, AsyncIterator

T = TypeVar("T")

class Clock(metaclass=ABCMeta):
    @abstractmethod
    def start_clock(self) -> None: ...
    @abstractmethod
    def current_time(self) -> float: ...
    @abstractmethod
    def deadline_to_sleep_time(self, deadline: float) -> float: ...

class Instrument(metaclass=ABCMeta):
    def before_run(self) -> None: ...
    def after_run(self) -> None: ...
    def task_spawned(self, task: trio.hazmat.Task) -> None: ...
    def task_scheduled(self, task: trio.hazmat.Task) -> None: ...
    def before_task_step(self, task: trio.hazmat.Task) -> None: ...
    def after_task_step(self, task: trio.hazmat.Task) -> None: ...
    def task_exited(self, task: trio.hazmat.Task) -> None: ...
    def before_io_wait(self, timeout: float) -> None: ...
    def after_io_wait(self, timeout: float) -> None: ...

class HostnameResolver(metaclass=ABCMeta):
    @abstractmethod
    async def getaddrinfo(
        self,
        host: Union[bytes, str],
        port: Union[str, int, None],
        family: int = ...,
        type: int = ...,
        proto: int = ...,
        flags: int = ...,
    ) -> List[Tuple[int, int, int, str, Tuple[Any, ...]]]: ...
    @abstractmethod
    async def getnameinfo(
        self, sockaddr: Tuple[Any, ...], flags: int
    ) -> Tuple[str, int]: ...

class SocketFactory(metaclass=ABCMeta):
    @abstractmethod
    def socket(
        self,
        family: Optional[int] = None,
        type: Optional[int] = None,
        proto: Optional[int] = None,
    ) -> trio.socket.SocketType: ...

class AsyncResource(metaclass=ABCMeta):
    @abstractmethod
    async def aclose(self) -> None: ...
    async def __aenter__(self: T) -> T: ...
    async def __aexit__(self, *exc: object) -> bool: ...

class SendStream(AsyncResource):
    @abstractmethod
    async def send_all(self, data: Union[bytes, memoryview]) -> None: ...
    @abstractmethod
    async def wait_send_all_might_not_block(self) -> None: ...

class ReceiveStream(AsyncResource):
    @abstractmethod
    async def receive_some(self, max_bytes: Optional[int] = ...) -> bytes: ...
    def __aiter__(self) -> AsyncIterator[bytes]: ...
    async def __anext__(self) -> bytes: ...

class Stream(SendStream, ReceiveStream, metaclass=ABCMeta):
    pass

class HalfCloseableStream(Stream):
    @abstractmethod
    async def send_eof(self) -> None: ...

_SomeResource = TypeVar("_SomeResource", bound=AsyncResource)

class Listener(AsyncResource, Generic[_SomeResource]):
    @abstractmethod
    async def accept(self) -> _SomeResource: ...

_T1 = TypeVar("_T1")

T_co = TypeVar("T_co", covariant=True)
T_contra = TypeVar("T_contra", contravariant=True)

class SendChannel(AsyncResource, Generic[T_contra]):
    @abstractmethod
    async def send(self, value: T_contra) -> None: ...

class ReceiveChannel(AsyncResource, Generic[T_co]):
    @abstractmethod
    async def receive(self) -> T_co: ...
    def __aiter__(self) -> AsyncIterator[T_co]: ...
    async def __anext__(self) -> T_co: ...

class Channel(SendChannel[T], ReceiveChannel[T], metaclass=ABCMeta):
    pass



import trio
from typing import Any, Awaitable, Callable, Optional, TypeVar, Union
from trio_typing import takes_callable_and_args
from mypy_extensions import VarArg

T = TypeVar("T")
@takes_callable_and_args
def run(
    afn: Union[Callable[..., Awaitable[T]], Callable[[VarArg()], Awaitable[T]]],
    *args: Any,
    trio_token: Optional[trio.hazmat.TrioToken] = ...,
) -> T: ...
@takes_callable_and_args
def run_sync(
    fn: Union[Callable[..., T], Callable[[VarArg()], T]],
    *args: Any,
    trio_token: Optional[trio.hazmat.TrioToken] = ...,
) -> T: ...



from typing import (
    Any,
    Awaitable,
    Callable,
    Generator,
    Generic,
    NoReturn,
    Optional,
    Type,
    TypeVar,
    Union,
)
from types import TracebackType
from typing_extensions import Protocol

T = TypeVar("T")
U = TypeVar("U")
T_co = TypeVar("T_co", covariant=True)
T_contra = TypeVar("T_contra", contravariant=True)

# Can't use AsyncGenerator as it creates a dependency cycle
# (outcome stubs -> trio_typing stubs -> trio.hazmat stubs -> outcome)
class _ASendable(Protocol[T_contra, T_co]):
    async def asend(self, value: T_contra) -> T_co: ...
    async def athrow(
        self,
        exc_type: Type[BaseException],
        exc_value: Optional[BaseException] = ...,
        exc_traceback: Optional[TracebackType] = ...,
    ) -> T_co: ...

class Value(Generic[T]):
    value: T
    def __init__(self, value: T): ...
    def unwrap(self) -> T: ...
    def send(self, gen: Generator[U, T, Any]) -> U: ...
    async def asend(self, gen: _ASendable[T, U]) -> U: ...

class Error:
    error: BaseException
    def __init__(self, error: BaseException): ...
    def unwrap(self) -> NoReturn: ...
    def send(self, gen: Generator[U, Any, Any]) -> U: ...
    async def asend(self, gen: _ASendable[Any, U]) -> U: ...

Outcome = Union[Value[T], Error]

# TODO: narrower typing for these (the args and kwargs should
# be acceptable to the callable)
def capture(sync_fn: Callable[..., T], *args: Any, **kwargs: Any) -> Outcome[T]: ...
async def acapture(
    async_fn: Callable[..., Awaitable[T]], *args: Any, **kwargs: Any
) -> Outcome[T]: ...



import os
import sys

if "trio_typing._tests.datadriven" not in sys.modules:

    def test_typecheck_dummy():
        import warnings

        warnings.warn(
            "Type-checking tests skipped because the plugin wasn't loaded. "
            "Run pytest with -p trio_typing._tests.datadriven to run them.",
            RuntimeWarning,
        )


else:
    from mypy import build
    from mypy.modulefinder import BuildSource
    from mypy.options import Options
    from mypy.test.data import DataDrivenTestCase, DataSuite
    from mypy.test.helpers import assert_string_arrays_equal

    class TrioTestSuite(DataSuite):
        data_prefix = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "test-data"
        )
        files = [name for name in os.listdir(data_prefix) if name.endswith(".test")]
        native_sep = True

        def run_case(self, testcase: DataDrivenTestCase) -> None:
            src = "\n".join(testcase.input)
            options = Options()
            options.show_traceback = True
            options.python_version = sys.version_info[:2]
            if testcase.name.endswith("_36"):
                options.python_version = (3, 6)
            else:
                options.python_version = sys.version_info[:2]
            if not testcase.name.endswith("_NoPlugin"):
                options.plugins = ["trio_typing.plugin"]
                # must specify something for config_file, else the
                # plugins don't get loaded
                options.config_file = "/dev/null"
            result = build.build(
                sources=[BuildSource("main", None, src)], options=options
            )
            assert_string_arrays_equal(
                testcase.output,
                result.errors,
                "Unexpected output from {0.file} line {0.line}".format(testcase),
            )



import sys
from typing import (
    Any,
    AsyncContextManager,
    AsyncIterable,
    AsyncIterator,
    Awaitable,
    Callable,
    Generic,
    NamedTuple,
    Optional,
    TypeVar,
    Union,
    overload,
)
from trio_typing import AsyncGenerator, CompatAsyncGenerator, YieldType, SendType
from typing_extensions import Protocol

_T = TypeVar("_T")

# The returned async generator's YieldType and SendType and the
# argument types of the decorated function get inferred by
# trio_typing.plugin
def async_generator(
    __fn: Callable[..., Awaitable[_T]]
) -> Callable[..., CompatAsyncGenerator[Any, Any, _T]]: ...

# The return type and a more specific argument type can be
# inferred by trio_typing.plugin, based on the enclosing
# @async_generator's YieldType and SendType
@overload
async def yield_() -> Any: ...
@overload
async def yield_(obj: object) -> Any: ...
@overload
async def yield_from_(agen: CompatAsyncGenerator[Any, Any, _T]) -> _T: ...
@overload
async def yield_from_(agen: AsyncGenerator[Any, Any]) -> None: ...
@overload
async def yield_from_(agen: AsyncIterable[Any]) -> None: ...
def isasyncgen(obj: object) -> bool: ...
def isasyncgenfunction(obj: object) -> bool: ...

# Argument types of the decorated function get inferred by
# trio_typing.plugin
def asynccontextmanager(
    fn: Callable[..., AsyncIterator[_T]]
) -> Callable[..., AsyncContextManager[_T]]: ...

class _AsyncCloseable(Protocol):
    async def aclose(self) -> None: ...

_T_closeable = TypeVar("_T_closeable", bound=_AsyncCloseable)

def aclosing(obj: _T_closeable) -> AsyncContextManager[_T_closeable]: ...

_AsyncGenHooks = NamedTuple(
    "AsyncGenHooks",
    [
        ("firstiter", Optional[Callable[[AsyncGenerator[Any, Any]], None]]),
        ("finalizer", Optional[Callable[[AsyncGenerator[Any, Any]], None]]),
    ],
)

def get_asyncgen_hooks() -> _AsyncGenHooks: ...
def set_asyncgen_hooks(
    firstiter: Optional[Callable[[AsyncGenerator[Any, Any]], None]] = ...,
    finalizer: Optional[Callable[[AsyncGenerator[Any, Any]], None]] = ...,
) -> None: ...



from abc import ABCMeta
from typing import (
    Any,
    AnyStr,
    AsyncContextManager,
    AsyncIterator,
    Awaitable,
    Callable,
    ContextManager,
    FrozenSet,
    Iterator,
    Mapping,
    NoReturn,
    Optional,
    Sequence,
    Union,
    Sequence,
    TypeVar,
    Tuple,
    List,
    Iterable,
    TextIO,
    BinaryIO,
    IO,
    overload,
)
from trio_typing import TaskStatus, takes_callable_and_args
from typing_extensions import Protocol, Literal
from mypy_extensions import NamedArg, VarArg
import attr
import signal
import io
import os
import pathlib
import subprocess
import ssl
import sys
import trio
from . import hazmat as hazmat, socket as socket, abc as abc
from . import to_thread as to_thread, from_thread as from_thread

T = TypeVar("T")
T_co = TypeVar("T_co", covariant=True)
T_contra = TypeVar("T_contra", contravariant=True)

class _Statistics:
    def __getattr__(self, name: str) -> Any: ...

# Inheriting from this (even outside of stubs) produces a class that
# mypy thinks is abstract, but the interpreter thinks is concrete.
class _NotConstructible(Protocol):
    _objects_of_this_type_are_not_directly_constructible: None

# _core._exceptions
class TrioInternalError(Exception):
    pass

class RunFinishedError(Exception):
    pass

class WouldBlock(Exception):
    pass

class BusyResourceError(Exception):
    pass

class ClosedResourceError(Exception):
    pass

class BrokenResourceError(Exception):
    pass

class EndOfChannel(Exception):
    pass

class NoHandshakeError(Exception):
    pass

class Cancelled(BaseException, _NotConstructible, metaclass=ABCMeta):
    pass

# _core._multierror
class MultiError(BaseException):
    exceptions: List[BaseException]
    def __init__(self, exceptions: Sequence[BaseException]): ...
    @classmethod
    def filter(
        cls,
        handler: Callable[[BaseException], Optional[BaseException]],
        root_exc: BaseException,
    ) -> BaseException: ...
    @classmethod
    def catch(
        cls, handler: Callable[[BaseException], Optional[BaseException]]
    ) -> ContextManager[None]: ...

# _core._run
TASK_STATUS_IGNORED: TaskStatus[Any]

class CancelScope:
    deadline: float
    shield: bool
    cancel_called: bool
    cancelled_caught: bool
    def __init__(self, *, deadline: float = ..., shield: bool = ...): ...
    def __enter__(self) -> CancelScope: ...
    def __exit__(self, *exc: object) -> bool: ...
    def cancel(self) -> None: ...

class Nursery(_NotConstructible, metaclass=ABCMeta):
    cancel_scope: CancelScope
    @property
    def child_tasks(self) -> FrozenSet[trio.hazmat.Task]: ...
    @property
    def parent_task(self) -> trio.hazmat.Task: ...
    @takes_callable_and_args
    def start_soon(
        self,
        async_fn: Union[
            # List these explicitly instead of Callable[..., Awaitable[None]]
            # so that even without the plugin we catch cases of passing a
            # function with keyword-only arguments to start_soon().
            Callable[[], Awaitable[None]],
            Callable[[Any], Awaitable[None]],
            Callable[[Any, Any], Awaitable[None]],
            Callable[[Any, Any, Any], Awaitable[None]],
            Callable[[Any, Any, Any, Any], Awaitable[None]],
            Callable[[VarArg()], Awaitable[None]],
        ],
        *args: Any,
        name: object = None,
    ) -> None: ...
    @takes_callable_and_args
    async def start(
        self,
        async_fn: Union[
            # List these explicitly instead of Callable[..., Awaitable[None]]
            # so that even without the plugin we can infer the return type
            # of start(), and fail when a function is passed that doesn't
            # accept task_status.
            Callable[[NamedArg(TaskStatus[T], "task_status")], Awaitable[None]],
            Callable[[Any, NamedArg(TaskStatus[T], "task_status")], Awaitable[None]],
            Callable[
                [Any, Any, NamedArg(TaskStatus[T], "task_status")], Awaitable[None]
            ],
            Callable[
                [Any, Any, Any, NamedArg(TaskStatus[T], "task_status")], Awaitable[None]
            ],
            Callable[
                [Any, Any, Any, Any, NamedArg(TaskStatus[T], "task_status")],
                Awaitable[None],
            ],
            Callable[
                [VarArg(), NamedArg(TaskStatus[T], "task_status")], Awaitable[None]
            ],
        ],
        *args: Any,
        name: object = None,
    ) -> T: ...

def open_nursery() -> AsyncContextManager[Nursery]: ...
def current_effective_deadline() -> float: ...
def current_time() -> float: ...
@takes_callable_and_args
def run(
    afn: Union[Callable[..., Awaitable[T]], Callable[[VarArg()], Awaitable[T]]],
    *args: Any,
    clock: trio.abc.Clock = ...,
    instruments: Sequence[trio.abc.Instrument] = ...,
    restrict_keyboard_interrupt_to_checkpoints: bool = ...,
) -> T: ...

# _timeouts
def move_on_at(deadline: float) -> CancelScope: ...
def move_on_after(seconds: float) -> CancelScope: ...
async def sleep_forever() -> NoReturn: ...
async def sleep_until(deadline: float) -> None: ...
async def sleep(seconds: float) -> None: ...
def fail_at(deadline: float) -> ContextManager[CancelScope]: ...
def fail_after(seconds: float) -> ContextManager[CancelScope]: ...

class TooSlowError(Exception):
    pass

# _sync
class Event:
    def is_set(self) -> bool: ...
    def set(self) -> None: ...
    async def wait(self) -> None: ...
    def statistics(self) -> _Statistics: ...

class CapacityLimiter:
    # float here really means Union[int, math.inf] but mypy doesn't
    # understand that
    total_tokens: float
    borrowed_tokens: int
    available_tokens: float
    def __init__(self, total_tokens: float): ...
    def acquire_nowait(self) -> None: ...
    def acquire_on_behalf_of_nowait(self, borrower: object) -> None: ...
    async def acquire(self) -> None: ...
    async def acquire_on_behalf_of(self, borrower: object) -> None: ...
    def release(self) -> None: ...
    def release_on_behalf_of(self, borrower: object) -> None: ...
    def statistics(self) -> _Statistics: ...
    async def __aenter__(self) -> None: ...
    async def __aexit__(self, *exc: object) -> bool: ...

class Semaphore:
    value: int
    max_value: Optional[int]
    def __init__(self, initial_value: int, *, max_value: Optional[int] = None): ...
    def acquire_nowait(self) -> None: ...
    async def acquire(self) -> None: ...
    def release(self) -> None: ...
    def statistics(self) -> _Statistics: ...
    async def __aenter__(self) -> None: ...
    async def __aexit__(self, *exc: object) -> bool: ...

class Lock:
    def locked(self) -> bool: ...
    def acquire_nowait(self) -> None: ...
    async def acquire(self) -> None: ...
    def release(self) -> None: ...
    def statistics(self) -> _Statistics: ...
    async def __aenter__(self) -> None: ...
    async def __aexit__(self, *exc: object) -> bool: ...

class StrictFIFOLock(Lock):
    pass

class Condition:
    def __init__(self, lock: Optional[Lock] = None) -> None: ...
    def locked(self) -> bool: ...
    def acquire_nowait(self) -> None: ...
    async def acquire(self) -> None: ...
    def release(self) -> None: ...
    async def wait(self) -> None: ...
    def notify(self, n: int = 1) -> None: ...
    def notify_all(self) -> None: ...
    def statistics(self) -> _Statistics: ...
    async def __aenter__(self) -> None: ...
    async def __aexit__(self, *exc: object) -> bool: ...

# _highlevel_generic
async def aclose_forcefully(resource: trio.abc.AsyncResource) -> None: ...

class StapledStream(trio.abc.HalfCloseableStream):
    send_stream: trio.abc.SendStream
    receive_stream: trio.abc.ReceiveStream
    def __init__(
        self, send_stream: trio.abc.SendStream, receive_stream: trio.abc.ReceiveStream
    ) -> None: ...
    async def aclose(self) -> None: ...
    async def send_all(self, data: Union[bytes, memoryview]) -> None: ...
    async def wait_send_all_might_not_block(self) -> None: ...
    async def receive_some(self, max_bytes: Optional[int] = ...) -> bytes: ...
    async def send_eof(self) -> None: ...

# _channel
class MemorySendChannel(trio.abc.SendChannel[T_contra]):
    def send_nowait(self, value: T_contra) -> None: ...
    async def send(self, value: T_contra) -> None: ...
    def clone(self: T) -> T: ...
    async def aclose(self) -> None: ...
    def statistics(self) -> _Statistics: ...

class MemoryReceiveChannel(trio.abc.ReceiveChannel[T_co]):
    def receive_nowait(self) -> T_co: ...
    async def receive(self) -> T_co: ...
    def clone(self: T) -> T: ...
    async def aclose(self) -> None: ...
    def statistics(self) -> _Statistics: ...

# written as a class so you can say open_memory_channel[int](5)
class open_memory_channel(Tuple[MemorySendChannel[T], MemoryReceiveChannel[T]]):
    def __new__(  # type: ignore[misc]  # "must return a subtype"
        cls, max_buffer_size: float
    ) -> Tuple[MemorySendChannel[T], MemoryReceiveChannel[T]]: ...
    def __init__(self, max_buffer_size: float): ...

# _signals
def open_signal_receiver(
    *signals: signal.Signals
) -> ContextManager[AsyncIterator[signal.Signals]]: ...

# _highlevel_socket
class SocketStream(trio.abc.HalfCloseableStream):
    socket: trio.socket.SocketType
    def __init__(self, socket: trio.socket.SocketType) -> None: ...
    def setsockopt(self, level: int, option: int, value: Union[int, bytes]) -> None: ...
    @overload
    def getsockopt(self, level: int, optname: int) -> int: ...
    @overload
    def getsockopt(self, level: int, optname: int, buflen: int) -> bytes: ...
    async def aclose(self) -> None: ...
    async def send_all(self, data: Union[bytes, memoryview]) -> None: ...
    async def wait_send_all_might_not_block(self) -> None: ...
    async def receive_some(self, max_bytes: Optional[int] = ...) -> bytes: ...
    async def send_eof(self) -> None: ...

class SocketListener(trio.abc.Listener[SocketStream]):
    socket: trio.socket.SocketType
    def __init__(self, socket: trio.socket.SocketType) -> None: ...
    async def accept(self) -> SocketStream: ...
    async def aclose(self) -> None: ...

# _file_io
class _AsyncIOBase(trio.abc.AsyncResource):
    closed: bool
    def __aiter__(self) -> AsyncIterator[bytes]: ...
    async def __anext__(self) -> bytes: ...
    async def aclose(self) -> None: ...
    def fileno(self) -> int: ...
    async def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def readable(self) -> bool: ...
    async def readlines(self, hint: int = ...) -> List[bytes]: ...
    async def seek(self, offset: int, whence: int = ...) -> int: ...
    def seekable(self) -> bool: ...
    async def tell(self) -> int: ...
    async def truncate(self, size: Optional[int] = ...) -> int: ...
    def writable(self) -> bool: ...
    async def writelines(self, lines: Iterable[bytes]) -> None: ...
    async def readline(self, size: int = ...) -> bytes: ...

class _AsyncRawIOBase(_AsyncIOBase):
    async def readall(self) -> bytes: ...
    async def readinto(self, b: bytearray) -> Optional[int]: ...
    async def write(self, b: bytes) -> Optional[int]: ...
    async def read(self, size: int = ...) -> Optional[bytes]: ...

class _AsyncBufferedIOBase(_AsyncIOBase):
    async def detach(self) -> _AsyncRawIOBase: ...
    async def readinto(self, b: bytearray) -> int: ...
    async def write(self, b: bytes) -> int: ...
    async def readinto1(self, b: bytearray) -> int: ...
    async def read(self, size: Optional[int] = ...) -> bytes: ...
    async def read1(self, size: int = ...) -> bytes: ...

class _AsyncTextIOBase(_AsyncIOBase):
    encoding: str
    errors: Optional[str]
    newlines: Union[str, Tuple[str, ...], None]
    def __aiter__(self) -> AsyncIterator[str]: ...  # type: ignore
    async def __anext__(self) -> str: ...  # type: ignore
    async def detach(self) -> _AsyncRawIOBase: ...
    async def write(self, s: str) -> int: ...
    async def readline(self, size: int = ...) -> str: ...  # type: ignore
    async def read(self, size: Optional[int] = ...) -> str: ...
    async def seek(self, offset: int, whence: int = ...) -> int: ...
    async def tell(self) -> int: ...

async def open_file(
    file: Union[str, bytes, int],
    mode: str = ...,
    buffering: int = ...,
    encoding: Optional[str] = ...,
    errors: Optional[str] = ...,
    newline: Optional[str] = ...,
    closefd: bool = ...,
    opener: Optional[Callable[[Union[int, str], str], int]] = ...,
) -> _AsyncIOBase: ...
@overload
def wrap_file(obj: Union[TextIO, io.TextIOBase]) -> _AsyncTextIOBase: ...
@overload
def wrap_file(obj: Union[BinaryIO, io.BufferedIOBase]) -> _AsyncBufferedIOBase: ...
@overload
def wrap_file(obj: io.RawIOBase) -> _AsyncRawIOBase: ...
@overload
def wrap_file(obj: Union[IO[Any], io.IOBase]) -> _AsyncIOBase: ...

# _path
class Path(pathlib.PurePath):
    @classmethod
    async def cwd(cls) -> Path: ...
    @classmethod
    async def home(cls) -> Path: ...
    async def stat(self) -> os.stat_result: ...
    async def chmod(self, mode: int) -> None: ...
    async def exists(self) -> bool: ...
    async def glob(self, pattern: str) -> Iterator[Path]: ...
    async def group(self) -> str: ...
    async def is_dir(self) -> bool: ...
    async def is_file(self) -> bool: ...
    async def is_symlink(self) -> bool: ...
    async def is_socket(self) -> bool: ...
    async def is_fifo(self) -> bool: ...
    async def is_block_device(self) -> bool: ...
    async def is_char_device(self) -> bool: ...
    async def iterdir(self) -> Iterator[Path]: ...
    async def lchmod(self, mode: int) -> None: ...
    async def lstat(self) -> os.stat_result: ...
    async def mkdir(
        self, mode: int = ..., parents: bool = ..., exist_ok: bool = ...
    ) -> None: ...
    async def open(
        self,
        mode: str = ...,
        buffering: int = ...,
        encoding: Optional[str] = ...,
        errors: Optional[str] = ...,
        newline: Optional[str] = ...,
    ) -> _AsyncIOBase: ...
    async def owner(self) -> str: ...
    async def rename(self, target: Union[str, pathlib.PurePath]) -> None: ...
    async def replace(self, target: Union[str, pathlib.PurePath]) -> None: ...
    if sys.version_info < (3, 6):
        async def resolve(self) -> Path: ...
    else:
        async def resolve(self, strict: bool = ...) -> Path: ...
    async def rglob(self, pattern: str) -> Iterator[Path]: ...
    async def rmdir(self) -> None: ...
    async def symlink_to(
        self, target: Union[str, pathlib.PurePath], target_is_directory: bool = ...
    ) -> None: ...
    async def touch(self, mode: int = ..., exist_ok: bool = ...) -> None: ...
    async def unlink(self) -> None: ...
    # added in 3.5:
    async def absolute(self) -> Path: ...
    async def expanduser(self) -> Path: ...
    async def read_bytes(self) -> bytes: ...
    async def read_text(
        self, encoding: Optional[str] = ..., errors: Optional[str] = ...
    ) -> str: ...
    async def samefile(self, other_path: Union[str, bytes, int, Path]) -> bool: ...
    async def write_bytes(self, data: bytes) -> int: ...
    async def write_text(
        self, data: str, encoding: Optional[str] = ..., errors: Optional[str] = ...
    ) -> int: ...

# _highlevel_serve_listeners
T_resource = TypeVar("T_resource", bound=trio.abc.AsyncResource)

async def serve_listeners(
    handler: Callable[[T_resource], Awaitable[None]],
    listeners: Sequence[trio.abc.Listener[T_resource]],
    *,
    handler_nursery: Optional[Nursery] = None,
    task_status: TaskStatus[Sequence[trio.abc.Listener[T_resource]]] = ...,
) -> NoReturn: ...

# _highlevel_open_tcp_stream
async def open_tcp_stream(
    host: AnyStr, port: int, *, happy_eyeballs_delay: float = ...
) -> SocketStream: ...

# _highlevel_open_tcp_listeners
async def open_tcp_listeners(
    port: int, *, host: Optional[AnyStr] = None, backlog: Optional[int] = None
) -> Sequence[SocketListener]: ...
async def serve_tcp(
    handler: Callable[[SocketStream], Awaitable[None]],
    port: int,
    *,
    host: Optional[AnyStr] = None,
    backlog: Optional[int] = None,
    handler_nursery: Optional[Nursery] = None,
    task_status: TaskStatus[Sequence[SocketListener]],
) -> NoReturn: ...

# _highlevel_open_unix_stream
async def open_unix_socket(filename: AnyStr) -> SocketStream: ...

# _highlevel_ssl_helpers
async def open_ssl_over_tcp_stream(
    host: AnyStr,
    port: int,
    *,
    https_compatible: bool = False,
    ssl_context: Optional[ssl.SSLContext] = None,
    happy_eyeballs_delay: float = ...,
) -> trio.SSLStream: ...
async def open_ssl_over_tcp_listeners(
    port: int,
    ssl_context: ssl.SSLContext,
    *,
    host: Optional[AnyStr] = None,
    https_compatible: bool = False,
    backlog: Optional[int] = None,
) -> Sequence[trio.SSLListener]: ...
async def serve_ssl_over_tcp(
    handler: Callable[[trio.SSLStream], Awaitable[None]],
    port: int,
    ssl_context: ssl.SSLContext,
    *,
    host: Optional[AnyStr] = None,
    https_compatible: bool = False,
    backlog: Optional[int] = None,
    handler_nursery: Optional[Nursery] = None,
    task_status: TaskStatus[Sequence[trio.SSLListener]],
) -> NoReturn: ...

# _ssl
class SSLStream(trio.abc.Stream):
    transport_stream: trio.abc.Stream
    context: ssl.SSLContext
    server_side: bool
    server_hostname: Optional[str]

    if sys.version_info >= (3, 6):
        @property
        def session(self) -> Optional[ssl.SSLSession]: ...
        @property
        def session_reused(self) -> bool: ...
    def __init__(
        self,
        transport_stream: trio.abc.Stream,
        ssl_context: ssl.SSLContext,
        *,
        server_hostname: Optional[str] = None,
        server_side: bool = False,
        https_compatible: bool = False,
        max_refill_bytes: int = ...,
    ) -> None: ...
    def getpeercert(self, binary_form: bool = ...) -> ssl._PeerCertRetType: ...
    def selected_npn_protocol(self) -> Optional[str]: ...
    def selected_alpn_protocol(self) -> Optional[str]: ...
    def cipher(self) -> Tuple[str, int, int]: ...
    def shared_ciphers(self) -> Optional[List[Tuple[str, int, int]]]: ...
    def compression(self) -> Optional[str]: ...
    def pending(self) -> int: ...
    def get_channel_binding(self, cb_type: str = ...) -> Optional[bytes]: ...
    async def do_handshake(self) -> None: ...
    async def unwrap(self) -> Tuple[trio.abc.Stream, bytes]: ...
    async def aclose(self) -> None: ...
    async def send_all(self, data: Union[bytes, memoryview]) -> None: ...
    async def wait_send_all_might_not_block(self) -> None: ...
    async def receive_some(self, max_bytes: Optional[int] = ...) -> bytes: ...

class SSLListener(trio.abc.Listener[SSLStream]):
    transport_listener: trio.abc.Listener[trio.abc.Stream]
    def __init__(
        self,
        transport_listener: trio.abc.Listener[trio.abc.Stream],
        ssl_context: ssl.SSLContext,
        *,
        https_compatible: bool = False,
        max_refill_bytes: int = ...,
    ) -> None: ...
    async def accept(self) -> SSLStream: ...
    async def aclose(self) -> None: ...

# _subprocess

class _HasFileno(Protocol):
    def fileno(self) -> int: ...

_Redirect = Union[int, _HasFileno, None]

class Process(trio.abc.AsyncResource, _NotConstructible, metaclass=ABCMeta):
    stdin: Optional[trio.abc.SendStream]
    stdout: Optional[trio.abc.ReceiveStream]
    stderr: Optional[trio.abc.ReceiveStream]
    stdio: Optional[StapledStream]
    args: Union[str, Sequence[str]]
    pid: int
    @property
    def returncode(self) -> Optional[int]: ...
    async def aclose(self) -> None: ...
    async def wait(self) -> int: ...
    def poll(self) -> Optional[int]: ...
    def send_signal(self, sig: signal.Signals) -> None: ...
    def terminate(self) -> None: ...
    def kill(self) -> None: ...

# There's a lot of duplication here because mypy doesn't
# have a good way to represent overloads that differ only
# slightly. A cheat sheet:
# - on Windows, command is Union[str, Sequence[str]];
#   on Unix, command is str if shell=True and Sequence[str] otherwise
# - on Windows, there are startupinfo and creationflags options;
#   on Unix, there are preexec_fn, restore_signals, start_new_session, and pass_fds
# - run_process() has the signature of open_process() plus arguments
#   capture_stdout, capture_stderr, check, and the ability to pass
#   bytes as stdin

if sys.platform == "win32":
    async def open_process(
        command: Union[str, Sequence[str]],
        *,
        stdin: _Redirect = ...,
        stdout: _Redirect = ...,
        stderr: _Redirect = ...,
        close_fds: bool = ...,
        shell: bool = ...,
        cwd: str = ...,
        env: Mapping[str, str] = ...,
        startupinfo: subprocess.STARTUPINFO = ...,
        creationflags: int = ...,
    ) -> Process: ...
    async def run_process(
        command: Union[str, Sequence[str]],
        *,
        stdin: Union[bytes, _Redirect] = ...,
        capture_stdout: bool = ...,
        capture_stderr: bool = ...,
        check: bool = ...,
        stdout: _Redirect = ...,
        stderr: _Redirect = ...,
        close_fds: bool = ...,
        shell: bool = ...,
        cwd: str = ...,
        env: Mapping[str, str] = ...,
        startupinfo: subprocess.STARTUPINFO = ...,
        creationflags: int = ...,
    ) -> subprocess.CompletedProcess[bytes]: ...

else:
    @overload
    async def open_process(
        command: str,
        *,
        stdin: _Redirect = ...,
        stdout: _Redirect = ...,
        stderr: _Redirect = ...,
        close_fds: bool = ...,
        shell: Literal[True],
        cwd: str = ...,
        env: Mapping[str, str] = ...,
        preexec_fn: Optional[Callable[[], None]] = ...,
        restore_signals: bool = ...,
        start_new_session: bool = ...,
        pass_fds: Sequence[int] = ...,
    ) -> Process: ...
    @overload
    async def open_process(
        command: Sequence[str],
        *,
        stdin: _Redirect = ...,
        stdout: _Redirect = ...,
        stderr: _Redirect = ...,
        close_fds: bool = ...,
        shell: bool = ...,
        cwd: str = ...,
        env: Mapping[str, str] = ...,
        preexec_fn: Optional[Callable[[], None]] = ...,
        restore_signals: bool = ...,
        start_new_session: bool = ...,
        pass_fds: Sequence[int] = ...,
    ) -> Process: ...
    @overload
    async def run_process(
        command: str,
        *,
        stdin: Union[bytes, _Redirect] = ...,
        capture_stdout: bool = ...,
        capture_stderr: bool = ...,
        check: bool = ...,
        stdout: _Redirect = ...,
        stderr: _Redirect = ...,
        close_fds: bool = ...,
        shell: Literal[True],
        cwd: str = ...,
        env: Mapping[str, str] = ...,
        preexec_fn: Optional[Callable[[], None]] = ...,
        restore_signals: bool = ...,
        start_new_session: bool = ...,
        pass_fds: Sequence[int] = ...,
    ) -> subprocess.CompletedProcess[bytes]: ...
    @overload
    async def run_process(
        command: Sequence[str],
        *,
        stdin: Union[bytes, _Redirect] = ...,
        capture_stdout: bool = ...,
        capture_stderr: bool = ...,
        check: bool = ...,
        stdout: _Redirect = ...,
        stderr: _Redirect = ...,
        close_fds: bool = ...,
        shell: bool = ...,
        cwd: str = ...,
        env: Mapping[str, str] = ...,
        preexec_fn: Optional[Callable[[], None]] = ...,
        restore_signals: bool = ...,
        start_new_session: bool = ...,
        pass_fds: Sequence[int] = ...,
    ) -> subprocess.CompletedProcess[bytes]: ...



import sys
import typing
import trio
import trio_typing
import async_generator


@async_generator.async_generator
async def compat_agen():
    pass


if sys.version_info >= (3, 6):
    exec("async def native_agen():\n  yield 42\n")
else:
    native_agen = compat_agen


def test_runtime():
    assert isinstance(compat_agen(), trio_typing.AsyncGenerator)
    assert isinstance(native_agen(), trio_typing.AsyncGenerator)
    if hasattr(typing, "AsyncGenerator"):
        assert trio_typing.AsyncGenerator is typing.AsyncGenerator

    async def task(*, task_status=trio.TASK_STATUS_IGNORED):
        assert isinstance(task_status, trio_typing.TaskStatus)
        task_status.started()

    async def main():
        async with trio.open_nursery() as nursery:
            assert isinstance(nursery, trio_typing.Nursery)
            nursery.start_soon(task)
            await nursery.start(task)



from setuptools import setup
import os

exec(open("trio_typing/_version.py", encoding="utf-8").read())

LONG_DESC = open("README.rst", encoding="utf-8").read()

stub_packages = ["async_generator-stubs", "outcome-stubs", "trio-stubs"]

setup(
    name="trio-typing",
    version=__version__,
    description="Static type checking support for Trio and related projects",
    url="https://github.com/python-trio/trio-typing",
    long_description=LONG_DESC,
    long_description_content_type="text/x-rst",
    author="Joshua Oreman",
    author_email="oremanj@gmail.com",
    license="MIT -or- Apache License 2.0",
    packages=["async_generator-stubs", "outcome-stubs", "trio-stubs", "trio_typing"],
    include_package_data=True,
    install_requires=[
        "trio >= 0.12.1",
        # mypy can't be installed on PyPy due to its dependency
        # on typed-ast
        "mypy >= 0.740; implementation_name == 'cpython'",
        "typing_extensions >= 3.7.4",
        "mypy_extensions >= 0.4.2",
    ],
    keywords=["async", "trio", "mypy"],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: Apache Software License",
        "Framework :: Trio",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: BSD",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
    ],
)



import abc as _abc
import sys as _sys
import typing as _t
import typing_extensions as _tx
import async_generator as _ag
import trio as _trio
from trio import Nursery
from ._version import __version__

__all__ = [
    "takes_callable_and_args",
    "Nursery",
    "TaskStatus",
    "AsyncGenerator",
    "CompatAsyncGenerator",
    "YieldType",
    "SendType",
]

_T = _t.TypeVar("_T")
_T_co = _t.TypeVar("_T_co", covariant=True)
_T_co2 = _t.TypeVar("_T_co2", covariant=True)
_T_contra = _t.TypeVar("_T_contra", contravariant=True)


def takes_callable_and_args(fn):
    return fn


class TaskStatus(_t.Generic[_T], metaclass=_abc.ABCMeta):
    pass


TaskStatus.register(_trio._core._run._TaskStatus)
TaskStatus.register(type(_trio.TASK_STATUS_IGNORED))

if _sys.version_info >= (3, 6):
    from typing import AsyncGenerator

else:

    class AsyncGenerator(_tx.AsyncIterator[_T_co], _t.Generic[_T_co, _T_contra]):
        pass


class CompatAsyncGenerator(
    AsyncGenerator[_T_co, _T_contra], _t.Generic[_T_co, _T_contra, _T_co2]
):
    pass


CompatAsyncGenerator.register(_ag._impl.AsyncGenerator)


class YieldType(_t.Generic[_T_co]):
    pass


class SendType(_t.Generic[_T_contra]):
    pass



from typing import (
    Union,
    AnyStr,
    Optional,
    Iterable,
    Tuple,
    List,
    Any,
    TypeVar,
    overload,
)
import sys
import trio

T = TypeVar("T")

import socket as _stdlib_socket
from socket import (
    AF_UNIX as AF_UNIX,
    AF_INET as AF_INET,
    AF_INET6 as AF_INET6,
    SOCK_STREAM as SOCK_STREAM,
    SOCK_DGRAM as SOCK_DGRAM,
    SOCK_RAW as SOCK_RAW,
    SOCK_RDM as SOCK_RDM,
    SOCK_SEQPACKET as SOCK_SEQPACKET,
    SOCK_CLOEXEC as SOCK_CLOEXEC,
    SOCK_NONBLOCK as SOCK_NONBLOCK,
    SOMAXCONN as SOMAXCONN,
    AF_AAL5 as AF_AAL5,
    AF_APPLETALK as AF_APPLETALK,
    AF_ASH as AF_ASH,
    AF_ATMPVC as AF_ATMPVC,
    AF_ATMSVC as AF_ATMSVC,
    AF_AX25 as AF_AX25,
    AF_BLUETOOTH as AF_BLUETOOTH,
    AF_BRIDGE as AF_BRIDGE,
    AF_CAN as AF_CAN,
    AF_DECnet as AF_DECnet,
    AF_ECONET as AF_ECONET,
    AF_IPX as AF_IPX,
    AF_IRDA as AF_IRDA,
    AF_KEY as AF_KEY,
    AF_LLC as AF_LLC,
    AF_NETBEUI as AF_NETBEUI,
    AF_NETLINK as AF_NETLINK,
    AF_NETROM as AF_NETROM,
    AF_PACKET as AF_PACKET,
    AF_PPPOX as AF_PPPOX,
    AF_RDS as AF_RDS,
    AF_ROSE as AF_ROSE,
    AF_ROUTE as AF_ROUTE,
    AF_SECURITY as AF_SECURITY,
    AF_SNA as AF_SNA,
    AF_SYSTEM as AF_SYSTEM,
    AF_TIPC as AF_TIPC,
    AF_UNSPEC as AF_UNSPEC,
    AF_WANPIPE as AF_WANPIPE,
    AF_X25 as AF_X25,
    AI_ADDRCONFIG as AI_ADDRCONFIG,
    AI_ALL as AI_ALL,
    AI_CANONNAME as AI_CANONNAME,
    AI_DEFAULT as AI_DEFAULT,
    AI_MASK as AI_MASK,
    AI_NUMERICHOST as AI_NUMERICHOST,
    AI_NUMERICSERV as AI_NUMERICSERV,
    AI_PASSIVE as AI_PASSIVE,
    AI_V4MAPPED as AI_V4MAPPED,
    AI_V4MAPPED_CFG as AI_V4MAPPED_CFG,
    BDADDR_ANY as BDADDR_ANY,
    BDADDR_LOCAL as BDADDR_LOCAL,
    BTPROTO_HCI as BTPROTO_HCI,
    BTPROTO_L2CAP as BTPROTO_L2CAP,
    BTPROTO_RFCOMM as BTPROTO_RFCOMM,
    BTPROTO_SCO as BTPROTO_SCO,
    CAN_EFF_FLAG as CAN_EFF_FLAG,
    CAN_EFF_MASK as CAN_EFF_MASK,
    CAN_ERR_FLAG as CAN_ERR_FLAG,
    CAN_ERR_MASK as CAN_ERR_MASK,
    CAN_RAW as CAN_RAW,
    CAN_RAW_ERR_FILTER as CAN_RAW_ERR_FILTER,
    CAN_RAW_FILTER as CAN_RAW_FILTER,
    CAN_RAW_LOOPBACK as CAN_RAW_LOOPBACK,
    CAN_RAW_RECV_OWN_MSGS as CAN_RAW_RECV_OWN_MSGS,
    CAN_RTR_FLAG as CAN_RTR_FLAG,
    CAN_SFF_MASK as CAN_SFF_MASK,
    CAPI as CAPI,
    EAGAIN as EAGAIN,
    EAI_ADDRFAMILY as EAI_ADDRFAMILY,
    EAI_AGAIN as EAI_AGAIN,
    EAI_BADFLAGS as EAI_BADFLAGS,
    EAI_BADHINTS as EAI_BADHINTS,
    EAI_FAIL as EAI_FAIL,
    EAI_FAMILY as EAI_FAMILY,
    EAI_MAX as EAI_MAX,
    EAI_MEMORY as EAI_MEMORY,
    EAI_NODATA as EAI_NODATA,
    EAI_NONAME as EAI_NONAME,
    EAI_OVERFLOW as EAI_OVERFLOW,
    EAI_PROTOCOL as EAI_PROTOCOL,
    EAI_SERVICE as EAI_SERVICE,
    EAI_SOCKTYPE as EAI_SOCKTYPE,
    EAI_SYSTEM as EAI_SYSTEM,
    EBADF as EBADF,
    EINTR as EINTR,
    EWOULDBLOCK as EWOULDBLOCK,
    HCI_DATA_DIR as HCI_DATA_DIR,
    HCI_FILTER as HCI_FILTER,
    HCI_TIME_STAMP as HCI_TIME_STAMP,
    INADDR_ALLHOSTS_GROUP as INADDR_ALLHOSTS_GROUP,
    INADDR_ANY as INADDR_ANY,
    INADDR_BROADCAST as INADDR_BROADCAST,
    INADDR_LOOPBACK as INADDR_LOOPBACK,
    INADDR_MAX_LOCAL_GROUP as INADDR_MAX_LOCAL_GROUP,
    INADDR_NONE as INADDR_NONE,
    INADDR_UNSPEC_GROUP as INADDR_UNSPEC_GROUP,
    IPPORT_RESERVED as IPPORT_RESERVED,
    IPPORT_USERRESERVED as IPPORT_USERRESERVED,
    IPPROTO_AH as IPPROTO_AH,
    IPPROTO_BIP as IPPROTO_BIP,
    IPPROTO_DSTOPTS as IPPROTO_DSTOPTS,
    IPPROTO_EGP as IPPROTO_EGP,
    IPPROTO_EON as IPPROTO_EON,
    IPPROTO_ESP as IPPROTO_ESP,
    IPPROTO_FRAGMENT as IPPROTO_FRAGMENT,
    IPPROTO_GGP as IPPROTO_GGP,
    IPPROTO_GRE as IPPROTO_GRE,
    IPPROTO_HELLO as IPPROTO_HELLO,
    IPPROTO_HOPOPTS as IPPROTO_HOPOPTS,
    IPPROTO_ICMP as IPPROTO_ICMP,
    IPPROTO_ICMPV6 as IPPROTO_ICMPV6,
    IPPROTO_IDP as IPPROTO_IDP,
    IPPROTO_IGMP as IPPROTO_IGMP,
    IPPROTO_IP as IPPROTO_IP,
    IPPROTO_IPCOMP as IPPROTO_IPCOMP,
    IPPROTO_IPIP as IPPROTO_IPIP,
    IPPROTO_IPV4 as IPPROTO_IPV4,
    IPPROTO_IPV6 as IPPROTO_IPV6,
    IPPROTO_MAX as IPPROTO_MAX,
    IPPROTO_MOBILE as IPPROTO_MOBILE,
    IPPROTO_ND as IPPROTO_ND,
    IPPROTO_NONE as IPPROTO_NONE,
    IPPROTO_PIM as IPPROTO_PIM,
    IPPROTO_PUP as IPPROTO_PUP,
    IPPROTO_RAW as IPPROTO_RAW,
    IPPROTO_ROUTING as IPPROTO_ROUTING,
    IPPROTO_RSVP as IPPROTO_RSVP,
    IPPROTO_SCTP as IPPROTO_SCTP,
    IPPROTO_TCP as IPPROTO_TCP,
    IPPROTO_TP as IPPROTO_TP,
    IPPROTO_UDP as IPPROTO_UDP,
    IPPROTO_VRRP as IPPROTO_VRRP,
    IPPROTO_XTP as IPPROTO_XTP,
    IPV6_CHECKSUM as IPV6_CHECKSUM,
    IPV6_DONTFRAG as IPV6_DONTFRAG,
    IPV6_DSTOPTS as IPV6_DSTOPTS,
    IPV6_HOPLIMIT as IPV6_HOPLIMIT,
    IPV6_HOPOPTS as IPV6_HOPOPTS,
    IPV6_JOIN_GROUP as IPV6_JOIN_GROUP,
    IPV6_LEAVE_GROUP as IPV6_LEAVE_GROUP,
    IPV6_MULTICAST_HOPS as IPV6_MULTICAST_HOPS,
    IPV6_MULTICAST_IF as IPV6_MULTICAST_IF,
    IPV6_MULTICAST_LOOP as IPV6_MULTICAST_LOOP,
    IPV6_NEXTHOP as IPV6_NEXTHOP,
    IPV6_PATHMTU as IPV6_PATHMTU,
    IPV6_PKTINFO as IPV6_PKTINFO,
    IPV6_RECVDSTOPTS as IPV6_RECVDSTOPTS,
    IPV6_RECVHOPLIMIT as IPV6_RECVHOPLIMIT,
    IPV6_RECVHOPOPTS as IPV6_RECVHOPOPTS,
    IPV6_RECVPATHMTU as IPV6_RECVPATHMTU,
    IPV6_RECVPKTINFO as IPV6_RECVPKTINFO,
    IPV6_RECVRTHDR as IPV6_RECVRTHDR,
    IPV6_RECVTCLASS as IPV6_RECVTCLASS,
    IPV6_RTHDR as IPV6_RTHDR,
    IPV6_RTHDR_TYPE_0 as IPV6_RTHDR_TYPE_0,
    IPV6_RTHDRDSTOPTS as IPV6_RTHDRDSTOPTS,
    IPV6_TCLASS as IPV6_TCLASS,
    IPV6_UNICAST_HOPS as IPV6_UNICAST_HOPS,
    IPV6_USE_MIN_MTU as IPV6_USE_MIN_MTU,
    IPV6_V6ONLY as IPV6_V6ONLY,
    IP_ADD_MEMBERSHIP as IP_ADD_MEMBERSHIP,
    IP_DEFAULT_MULTICAST_LOOP as IP_DEFAULT_MULTICAST_LOOP,
    IP_DEFAULT_MULTICAST_TTL as IP_DEFAULT_MULTICAST_TTL,
    IP_DROP_MEMBERSHIP as IP_DROP_MEMBERSHIP,
    IP_HDRINCL as IP_HDRINCL,
    IP_MAX_MEMBERSHIPS as IP_MAX_MEMBERSHIPS,
    IP_MULTICAST_IF as IP_MULTICAST_IF,
    IP_MULTICAST_LOOP as IP_MULTICAST_LOOP,
    IP_MULTICAST_TTL as IP_MULTICAST_TTL,
    IP_OPTIONS as IP_OPTIONS,
    IP_RECVDSTADDR as IP_RECVDSTADDR,
    IP_RECVOPTS as IP_RECVOPTS,
    IP_RECVRETOPTS as IP_RECVRETOPTS,
    IP_RETOPTS as IP_RETOPTS,
    IP_TOS as IP_TOS,
    IP_TRANSPARENT as IP_TRANSPARENT,
    IP_TTL as IP_TTL,
    IPX_TYPE as IPX_TYPE,
    LOCAL_PEERCRED as LOCAL_PEERCRED,
    MSG_BCAST as MSG_BCAST,
    MSG_BTAG as MSG_BTAG,
    MSG_CMSG_CLOEXEC as MSG_CMSG_CLOEXEC,
    MSG_CONFIRM as MSG_CONFIRM,
    MSG_CTRUNC as MSG_CTRUNC,
    MSG_DONTROUTE as MSG_DONTROUTE,
    MSG_DONTWAIT as MSG_DONTWAIT,
    MSG_EOF as MSG_EOF,
    MSG_EOR as MSG_EOR,
    MSG_ERRQUEUE as MSG_ERRQUEUE,
    MSG_ETAG as MSG_ETAG,
    MSG_FASTOPEN as MSG_FASTOPEN,
    MSG_MCAST as MSG_MCAST,
    MSG_MORE as MSG_MORE,
    MSG_NOSIGNAL as MSG_NOSIGNAL,
    MSG_NOTIFICATION as MSG_NOTIFICATION,
    MSG_OOB as MSG_OOB,
    MSG_PEEK as MSG_PEEK,
    MSG_TRUNC as MSG_TRUNC,
    MSG_WAITALL as MSG_WAITALL,
    NETLINK_ARPD as NETLINK_ARPD,
    NETLINK_CRYPTO as NETLINK_CRYPTO,
    NETLINK_DNRTMSG as NETLINK_DNRTMSG,
    NETLINK_FIREWALL as NETLINK_FIREWALL,
    NETLINK_IP6_FW as NETLINK_IP6_FW,
    NETLINK_NFLOG as NETLINK_NFLOG,
    NETLINK_ROUTE6 as NETLINK_ROUTE6,
    NETLINK_ROUTE as NETLINK_ROUTE,
    NETLINK_SKIP as NETLINK_SKIP,
    NETLINK_TAPBASE as NETLINK_TAPBASE,
    NETLINK_TCPDIAG as NETLINK_TCPDIAG,
    NETLINK_USERSOCK as NETLINK_USERSOCK,
    NETLINK_W1 as NETLINK_W1,
    NETLINK_XFRM as NETLINK_XFRM,
    NI_DGRAM as NI_DGRAM,
    NI_MAXHOST as NI_MAXHOST,
    NI_MAXSERV as NI_MAXSERV,
    NI_NAMEREQD as NI_NAMEREQD,
    NI_NOFQDN as NI_NOFQDN,
    NI_NUMERICHOST as NI_NUMERICHOST,
    NI_NUMERICSERV as NI_NUMERICSERV,
    PACKET_BROADCAST as PACKET_BROADCAST,
    PACKET_FASTROUTE as PACKET_FASTROUTE,
    PACKET_HOST as PACKET_HOST,
    PACKET_LOOPBACK as PACKET_LOOPBACK,
    PACKET_MULTICAST as PACKET_MULTICAST,
    PACKET_OTHERHOST as PACKET_OTHERHOST,
    PACKET_OUTGOING as PACKET_OUTGOING,
    PF_CAN as PF_CAN,
    PF_PACKET as PF_PACKET,
    PF_RDS as PF_RDS,
    PF_SYSTEM as PF_SYSTEM,
    SCM_CREDENTIALS as SCM_CREDENTIALS,
    SCM_CREDS as SCM_CREDS,
    SCM_RIGHTS as SCM_RIGHTS,
    SHUT_RD as SHUT_RD,
    SHUT_RDWR as SHUT_RDWR,
    SHUT_WR as SHUT_WR,
    SOL_ATALK as SOL_ATALK,
    SOL_AX25 as SOL_AX25,
    SOL_CAN_BASE as SOL_CAN_BASE,
    SOL_CAN_RAW as SOL_CAN_RAW,
    SOL_HCI as SOL_HCI,
    SOL_IP as SOL_IP,
    SOL_IPX as SOL_IPX,
    SOL_NETROM as SOL_NETROM,
    SOL_RDS as SOL_RDS,
    SOL_ROSE as SOL_ROSE,
    SOL_SOCKET as SOL_SOCKET,
    SOL_TCP as SOL_TCP,
    SOL_TIPC as SOL_TIPC,
    SOL_UDP as SOL_UDP,
    SO_ACCEPTCONN as SO_ACCEPTCONN,
    SO_BINDTODEVICE as SO_BINDTODEVICE,
    SO_BROADCAST as SO_BROADCAST,
    SO_DEBUG as SO_DEBUG,
    SO_DONTROUTE as SO_DONTROUTE,
    SO_ERROR as SO_ERROR,
    SO_EXCLUSIVEADDRUSE as SO_EXCLUSIVEADDRUSE,
    SO_KEEPALIVE as SO_KEEPALIVE,
    SO_LINGER as SO_LINGER,
    SO_MARK as SO_MARK,
    SO_OOBINLINE as SO_OOBINLINE,
    SO_PASSCRED as SO_PASSCRED,
    SO_PEERCRED as SO_PEERCRED,
    SO_PRIORITY as SO_PRIORITY,
    SO_RCVBUF as SO_RCVBUF,
    SO_RCVLOWAT as SO_RCVLOWAT,
    SO_RCVTIMEO as SO_RCVTIMEO,
    SO_REUSEADDR as SO_REUSEADDR,
    SO_REUSEPORT as SO_REUSEPORT,
    SO_SETFIB as SO_SETFIB,
    SO_SNDBUF as SO_SNDBUF,
    SO_SNDLOWAT as SO_SNDLOWAT,
    SO_SNDTIMEO as SO_SNDTIMEO,
    SO_TYPE as SO_TYPE,
    SO_USELOOPBACK as SO_USELOOPBACK,
    SYSPROTO_CONTROL as SYSPROTO_CONTROL,
    TCP_CORK as TCP_CORK,
    TCP_DEFER_ACCEPT as TCP_DEFER_ACCEPT,
    TCP_FASTOPEN as TCP_FASTOPEN,
    TCP_INFO as TCP_INFO,
    TCP_KEEPCNT as TCP_KEEPCNT,
    TCP_KEEPIDLE as TCP_KEEPIDLE,
    TCP_KEEPINTVL as TCP_KEEPINTVL,
    TCP_LINGER2 as TCP_LINGER2,
    TCP_MAXSEG as TCP_MAXSEG,
    TCP_NODELAY as TCP_NODELAY,
    TCP_NOTSENT_LOWAT as TCP_NOTSENT_LOWAT,
    TCP_QUICKACK as TCP_QUICKACK,
    TCP_SYNCNT as TCP_SYNCNT,
    TCP_WINDOW_CLAMP as TCP_WINDOW_CLAMP,
    TIPC_ADDR_ID as TIPC_ADDR_ID,
    TIPC_ADDR_NAME as TIPC_ADDR_NAME,
    TIPC_ADDR_NAMESEQ as TIPC_ADDR_NAMESEQ,
    TIPC_CFG_SRV as TIPC_CFG_SRV,
    TIPC_CLUSTER_SCOPE as TIPC_CLUSTER_SCOPE,
    TIPC_CONN_TIMEOUT as TIPC_CONN_TIMEOUT,
    TIPC_CRITICAL_IMPORTANCE as TIPC_CRITICAL_IMPORTANCE,
    TIPC_DEST_DROPPABLE as TIPC_DEST_DROPPABLE,
    TIPC_HIGH_IMPORTANCE as TIPC_HIGH_IMPORTANCE,
    TIPC_IMPORTANCE as TIPC_IMPORTANCE,
    TIPC_LOW_IMPORTANCE as TIPC_LOW_IMPORTANCE,
    TIPC_MEDIUM_IMPORTANCE as TIPC_MEDIUM_IMPORTANCE,
    TIPC_NODE_SCOPE as TIPC_NODE_SCOPE,
    TIPC_PUBLISHED as TIPC_PUBLISHED,
    TIPC_SRC_DROPPABLE as TIPC_SRC_DROPPABLE,
    TIPC_SUB_CANCEL as TIPC_SUB_CANCEL,
    TIPC_SUB_PORTS as TIPC_SUB_PORTS,
    TIPC_SUB_SERVICE as TIPC_SUB_SERVICE,
    TIPC_SUBSCR_TIMEOUT as TIPC_SUBSCR_TIMEOUT,
    TIPC_TOP_SRV as TIPC_TOP_SRV,
    TIPC_WAIT_FOREVER as TIPC_WAIT_FOREVER,
    TIPC_WITHDRAWN as TIPC_WITHDRAWN,
    TIPC_ZONE_SCOPE as TIPC_ZONE_SCOPE,
    RDS_CANCEL_SENT_TO as RDS_CANCEL_SENT_TO,
    RDS_CMSG_RDMA_ARGS as RDS_CMSG_RDMA_ARGS,
    RDS_CMSG_RDMA_DEST as RDS_CMSG_RDMA_DEST,
    RDS_CMSG_RDMA_MAP as RDS_CMSG_RDMA_MAP,
    RDS_CMSG_RDMA_STATUS as RDS_CMSG_RDMA_STATUS,
    RDS_CMSG_RDMA_UPDATE as RDS_CMSG_RDMA_UPDATE,
    RDS_CONG_MONITOR as RDS_CONG_MONITOR,
    RDS_FREE_MR as RDS_FREE_MR,
    RDS_GET_MR as RDS_GET_MR,
    RDS_GET_MR_FOR_DEST as RDS_GET_MR_FOR_DEST,
    RDS_RDMA_DONTWAIT as RDS_RDMA_DONTWAIT,
    RDS_RDMA_FENCE as RDS_RDMA_FENCE,
    RDS_RDMA_INVALIDATE as RDS_RDMA_INVALIDATE,
    RDS_RDMA_NOTIFY_ME as RDS_RDMA_NOTIFY_ME,
    RDS_RDMA_READWRITE as RDS_RDMA_READWRITE,
    RDS_RDMA_SILENT as RDS_RDMA_SILENT,
    RDS_RDMA_USE_ONCE as RDS_RDMA_USE_ONCE,
    RDS_RECVERR as RDS_RECVERR,
    CAN_BCM as CAN_BCM,
    CAN_BCM_TX_SETUP as CAN_BCM_TX_SETUP,
    CAN_BCM_TX_DELETE as CAN_BCM_TX_DELETE,
    CAN_BCM_TX_READ as CAN_BCM_TX_READ,
    CAN_BCM_TX_SEND as CAN_BCM_TX_SEND,
    CAN_BCM_RX_SETUP as CAN_BCM_RX_SETUP,
    CAN_BCM_RX_DELETE as CAN_BCM_RX_DELETE,
    CAN_BCM_RX_READ as CAN_BCM_RX_READ,
    CAN_BCM_TX_STATUS as CAN_BCM_TX_STATUS,
    CAN_BCM_TX_EXPIRED as CAN_BCM_TX_EXPIRED,
    CAN_BCM_RX_STATUS as CAN_BCM_RX_STATUS,
    CAN_BCM_RX_TIMEOUT as CAN_BCM_RX_TIMEOUT,
    CAN_BCM_RX_CHANGED as CAN_BCM_RX_CHANGED,
    AF_LINK as AF_LINK,
    CAN_RAW_FD_FRAMES as CAN_RAW_FD_FRAMES,
)

if sys.version_info >= (3, 6):
    from socket import (
        SO_DOMAIN as SO_DOMAIN,
        SO_PROTOCOL as SO_PROTOCOL,
        SO_PEERSEC as SO_PEERSEC,
        SO_PASSSEC as SO_PASSSEC,
        TCP_USER_TIMEOUT as TCP_USER_TIMEOUT,
        TCP_CONGESTION as TCP_CONGESTION,
        AF_ALG as AF_ALG,
        SOL_ALG as SOL_ALG,
        ALG_SET_KEY as ALG_SET_KEY,
        ALG_SET_IV as ALG_SET_IV,
        ALG_SET_OP as ALG_SET_OP,
        ALG_SET_AEAD_ASSOCLEN as ALG_SET_AEAD_ASSOCLEN,
        ALG_SET_AEAD_AUTHSIZE as ALG_SET_AEAD_AUTHSIZE,
        ALG_SET_PUBKEY as ALG_SET_PUBKEY,
        ALG_OP_DECRYPT as ALG_OP_DECRYPT,
        ALG_OP_ENCRYPT as ALG_OP_ENCRYPT,
        ALG_OP_SIGN as ALG_OP_SIGN,
        ALG_OP_VERIFY as ALG_OP_VERIFY,
    )

from socket import (
    gaierror as gaierror,
    herror as herror,
    gethostname as gethostname,
    ntohl as ntohl,
    ntohs as ntohs,
    htonl as htonl,
    htons as htons,
    inet_aton as inet_aton,
    inet_ntoa as inet_ntoa,
    inet_pton as inet_pton,
    inet_ntop as inet_ntop,
    sethostname as sethostname,
    if_nameindex as if_nameindex,
    if_nametoindex as if_nametoindex,
    if_indextoname as if_indextoname,
)

def set_custom_hostname_resolver(
    hostname_resolver: Optional[trio.abc.HostnameResolver]
) -> Optional[trio.abc.HostnameResolver]: ...
def set_custom_socket_factory(
    socket_factory: Optional[trio.abc.SocketFactory]
) -> Optional[trio.abc.SocketFactory]: ...
async def getnameinfo(sockaddr: Tuple[Any, ...], flags: int) -> Tuple[str, int]: ...
async def getprotobyname(name: str) -> int: ...
async def getaddrinfo(
    host: Union[bytearray, bytes, str],
    port: Union[str, int, None],
    family: int = ...,
    type: int = ...,
    proto: int = ...,
    flags: int = ...,
) -> List[Tuple[int, int, int, str, Tuple[Any, ...]]]: ...

class SocketType:
    family: int
    type: int
    proto: int
    did_shutdown_SHUT_WR: bool
    def __enter__(self: T) -> T: ...
    def __exit__(self, *args: Any) -> None: ...
    def dup(self) -> SocketType: ...
    def close(self) -> None: ...
    async def bind(self, address: Union[Tuple[Any, ...], str, bytes]) -> None: ...
    def shutdown(self, flag: int) -> None: ...
    def is_readable(self) -> bool: ...
    async def wait_writable(self) -> None: ...
    async def accept(self) -> Tuple[SocketType, Any]: ...
    async def connect(self, address: Union[Tuple[Any, ...], str, bytes]) -> None: ...
    async def recv(self, bufsize: int, flags: int = ...) -> bytes: ...
    async def recv_into(
        self, buffer: Union[bytearray, memoryview], nbytes: int, flags: int = ...
    ) -> int: ...
    async def recvfrom(self, bufsize: int, flags: int = ...) -> Tuple[bytes, Any]: ...
    async def recvfrom_into(
        self, buffer: Union[bytearray, memoryview], nbytes: int, flags: int = ...
    ) -> Tuple[int, Any]: ...
    async def recvmsg(
        self, bufsize: int, ancbufsize: int = ..., flags: int = ...
    ) -> Tuple[bytes, List[Tuple[int, int, bytes]], int, Any]: ...
    async def recvmsg_into(
        self,
        buffers: Iterable[Union[bytearray, memoryview]],
        ancbufsize: int = ...,
        flags: int = ...,
    ) -> Tuple[int, List[Tuple[int, int, bytes]], int, Any]: ...
    async def send(self, data: bytes, flags: int = ...) -> int: ...
    async def sendmsg(
        self,
        buffers: Iterable[Union[bytes, memoryview]],
        ancdata: Iterable[Tuple[int, int, Union[bytes, memoryview]]] = ...,
        flags: int = ...,
        address: Union[Tuple[Any, ...], str] = ...,
    ) -> int: ...
    @overload
    async def sendto(
        self, data: bytes, address: Union[Tuple[Any, ...], str]
    ) -> int: ...
    @overload
    async def sendto(
        self, data: bytes, flags: int, address: Union[Tuple[Any, ...], str]
    ) -> int: ...
    def detach(self) -> int: ...
    def get_inheritable(self) -> bool: ...
    def set_inheritable(self, inheritable: bool) -> None: ...
    def fileno(self) -> int: ...
    def getpeername(self) -> Union[None, Tuple[Any, ...], str]: ...
    def getsockname(self) -> Union[None, Tuple[Any, ...], str]: ...
    @overload
    def getsockopt(self, level: int, optname: int) -> int: ...
    @overload
    def getsockopt(self, level: int, optname: int, buflen: int) -> bytes: ...
    def setsockopt(
        self, level: int, optname: int, value: Union[int, bytes]
    ) -> None: ...
    def listen(self, backlog: int) -> None: ...
    def share(self, process_id: int) -> bytes: ...

def fromfd(fd: int, family: int, type: int, proto: int = ...) -> SocketType: ...
def fromshare(data: bytes) -> SocketType: ...
def from_stdlib_socket(sock: _stdlib_socket.socket) -> SocketType: ...
def socketpair(
    family: int = ..., type: int = ..., proto: int = ...
) -> Tuple[SocketType, SocketType]: ...
def socket(
    family: int = ..., type: int = ..., proto: int = ..., fileno: Optional[int] = ...
) -> SocketType: ...



import sys
from typing import Callable, List, Optional, Sequence, Tuple, cast
from typing_extensions import Literal
from typing import Type as typing_Type
from mypy.plugin import Plugin, FunctionContext, MethodContext, CheckerPluginInterface
from mypy.nodes import (
    ARG_POS,
    ARG_STAR,
    TypeInfo,
    Context,
    FuncDef,
    StrExpr,
    IntExpr,
    Expression,
)
from mypy.types import (
    Type,
    CallableType,
    NoneTyp,
    Overloaded,
    TypeVarDef,
    TypeVarType,
    Instance,
    UnionType,
    UninhabitedType,
    AnyType,
    TypeOfAny,
)
from mypy.typeops import make_simplified_union
from mypy.checker import TypeChecker


class TrioPlugin(Plugin):
    def get_function_hook(
        self, fullname: str
    ) -> Optional[Callable[[FunctionContext], Type]]:
        if fullname in (
            "contextlib.asynccontextmanager",
            "async_generator.asynccontextmanager",
        ):
            return args_invariant_decorator_callback
        if fullname == "trio.open_file":
            return open_file_callback
        if fullname == "trio_typing.takes_callable_and_args":
            return takes_callable_and_args_callback
        if fullname == "async_generator.async_generator":
            return async_generator_callback
        if fullname == "async_generator.yield_":
            return yield_callback
        if fullname == "async_generator.yield_from_":
            return yield_from_callback
        return None

    def get_method_hook(
        self, fullname: str
    ) -> Optional[Callable[[MethodContext], Type]]:
        if fullname == "trio_typing.TaskStatus.started":
            return started_callback
        if fullname == "trio.Path.open":
            return open_method_callback
        return None


def args_invariant_decorator_callback(ctx: FunctionContext) -> Type:
    """Infer a better return type for @asynccontextmanager,
    @async_generator, and other decorators that affect the return
    type but not the argument types of the function they decorate.
    """
    # (adapted from the @contextmanager support in mypy's builtin plugin)
    if ctx.arg_types and len(ctx.arg_types[0]) == 1:
        arg_type = ctx.arg_types[0][0]
        if isinstance(arg_type, CallableType) and isinstance(
            ctx.default_return_type, CallableType
        ):
            return ctx.default_return_type.copy_modified(
                arg_types=arg_type.arg_types,
                arg_kinds=arg_type.arg_kinds,
                arg_names=arg_type.arg_names,
                variables=arg_type.variables,
                is_ellipsis_args=arg_type.is_ellipsis_args,
            )
    return ctx.default_return_type


def open_return_type(
    api: CheckerPluginInterface, args: List[List[Expression]]
) -> Optional[Type]:
    def return_type(word: Literal["Text", "Buffered", "Raw"]) -> Type:
        return api.named_generic_type(
            "typing.Awaitable",
            [api.named_generic_type("trio._Async{}IOBase".format(word), [])],
        )

    if len(args) < 2 or len(args[1]) == 0:
        # If mode is unspecified, the default is text
        return return_type("Text")

    if len(args[1]) == 1:
        # Mode was specified
        mode_arg = args[1][0]
        if isinstance(mode_arg, StrExpr):
            # Mode is a string constant
            if "b" not in mode_arg.value:
                # If there's no "b" in it, it's a text mode
                return return_type("Text")
            # Otherwise it's binary -- determine whether buffered or not
            if len(args) >= 3 and len(args[2]) == 1:
                # Buffering was specified
                buffering_arg = args[2][0]
                if isinstance(buffering_arg, IntExpr):
                    # Buffering is a constant -- zero means
                    # unbuffered, otherwise buffered
                    if buffering_arg.value == 0:
                        return return_type("Raw")
                    return return_type("Buffered")
                # Not a constant, so we're not sure which it is.
                options = [
                    api.named_generic_type("trio._AsyncRawIOBase", []),
                    api.named_generic_type("trio._AsyncBufferedIOBase", []),
                ]  # type: List[Type]
                return api.named_generic_type(
                    "typing.Awaitable", [make_simplified_union(options)]
                )
            else:
                # Buffering is default if not specified
                return return_type("Buffered")

    # Mode wasn't a constant or we couldn't make sense of it
    return None


def open_file_callback(ctx: FunctionContext) -> Type:
    """Infer a better return type for trio.open_file()."""
    return open_return_type(ctx.api, ctx.args) or ctx.default_return_type


def open_method_callback(ctx: MethodContext) -> Type:
    """Infer a better return type for trio.Path.open()."""

    # Path.open() doesn't take the first (filename) argument of open_file(),
    # so we need to shift by one.
    args_with_path = cast(List[List[Expression]], [[]]) + ctx.args
    return open_return_type(ctx.api, args_with_path) or ctx.default_return_type


def decode_agen_types_from_return_type(
    ctx: FunctionContext, original_async_return_type: Type
) -> Tuple[Type, Type, Type]:
    """Return the yield type, send type, and return type of
    an @async_generator decorated function that was
    originally declared to return ``original_async_return_type``.

    This tries to interpret ``original_async_return_type`` as a union
    between the async generator return type (i.e., the thing actually
    returned by the decorated function, which becomes the value
    associated with a ``StopAsyncIteration`` exception), an optional
    ``trio_typing.YieldType[X]`` where ``X`` is the type of values
    that the async generator yields, and an optional
    ``trio_typing.SendType[Y]`` where ``Y`` is the type of values that
    the async generator expects to be sent. If one of ``YieldType``
    and ``SendType`` is specified, the other is assumed to be None;
    if neither is specified, both are assumed to be Any.
    If ``original_async_return_type`` includes a ``YieldType``
    and/or a ``SendType`` but no actual return type, the return
    is inferred as ``NoReturn``.
    """

    arms = []  # type: Sequence[Type]
    if isinstance(original_async_return_type, UnionType):
        arms = original_async_return_type.items
    else:
        arms = [original_async_return_type]
    yield_type = None  # type: Optional[Type]
    send_type = None  # type: Optional[Type]
    other_arms = []  # type: List[Type]
    try:
        for arm in arms:
            if isinstance(arm, Instance):
                if arm.type.fullname() == "trio_typing.YieldType":
                    if len(arm.args) != 1:
                        raise ValueError("YieldType must take one argument")
                    if yield_type is not None:
                        raise ValueError("YieldType specified multiple times")
                    yield_type = arm.args[0]
                elif arm.type.fullname() == "trio_typing.SendType":
                    if len(arm.args) != 1:
                        raise ValueError("SendType must take one argument")
                    if send_type is not None:
                        raise ValueError("SendType specified multiple times")
                    send_type = arm.args[0]
                else:
                    other_arms.append(arm)
            else:
                other_arms.append(arm)
    except ValueError as ex:
        ctx.api.fail("invalid @async_generator return type: {}".format(ex), ctx.context)
        return (
            AnyType(TypeOfAny.from_error),
            AnyType(TypeOfAny.from_error),
            original_async_return_type,
        )

    if yield_type is None and send_type is None:
        return (
            AnyType(TypeOfAny.unannotated),
            AnyType(TypeOfAny.unannotated),
            original_async_return_type,
        )

    if yield_type is None:
        yield_type = NoneTyp(ctx.context.line, ctx.context.column)
    if send_type is None:
        send_type = NoneTyp(ctx.context.line, ctx.context.column)
    if not other_arms:
        return (
            yield_type,
            send_type,
            UninhabitedType(
                is_noreturn=True, line=ctx.context.line, column=ctx.context.column
            ),
        )
    else:
        return (
            yield_type,
            send_type,
            make_simplified_union(other_arms, ctx.context.line, ctx.context.column),
        )


def async_generator_callback(ctx: FunctionContext) -> Type:
    """Handle @async_generator.

    This moves the yield type and send type declarations from
    the return type of the decorated function to the appropriate
    type parameters of ``trio_typing.CompatAsyncGenerator``.
    That is, if you say::

        @async_generator
        async def example() -> Union[str, YieldType[bool], SendType[int]]:
            ...

    then the decorated ``example()`` will return values of type
    ``CompatAsyncGenerator[bool, int, str]``, as opposed to
    ``CompatAsyncGenerator[Any, Any, Union[str,
    YieldType[bool], SendType[int]]`` without the plugin.
    """

    # Apply the common logic to not change the arguments of the
    # decorated function
    new_return_type = args_invariant_decorator_callback(ctx)
    if (
        isinstance(new_return_type, CallableType)
        and isinstance(new_return_type.ret_type, Instance)
        and new_return_type.ret_type.type.fullname()
        == ("trio_typing.CompatAsyncGenerator")
        and len(new_return_type.ret_type.args) == 3
    ):
        return new_return_type.copy_modified(
            ret_type=new_return_type.ret_type.copy_modified(
                args=list(
                    decode_agen_types_from_return_type(
                        ctx, new_return_type.ret_type.args[2]
                    )
                )
            )
        )
    return new_return_type


def decode_enclosing_agen_types(ctx: FunctionContext) -> Tuple[Type, Type]:
    """Return the yield and send types that would be returned by
    decode_agen_types_from_return_type() for the function that's
    currently being typechecked, i.e., the function that contains the
    call described in ``ctx``.
    """
    private_api = cast(TypeChecker, ctx.api)
    enclosing_func = private_api.scope.top_function()
    if (
        enclosing_func is None
        or not isinstance(enclosing_func, FuncDef)
        or not enclosing_func.is_coroutine
        or not enclosing_func.is_decorated
    ):
        # we can't actually detect the @async_generator decorator but
        # we'll at least notice if it couldn't possibly be present
        ctx.api.fail(
            "async_generator.yield_() outside an @async_generator func", ctx.context
        )
        return AnyType(TypeOfAny.from_error), AnyType(TypeOfAny.from_error)

    if (
        isinstance(enclosing_func.type, CallableType)
        and isinstance(enclosing_func.type.ret_type, Instance)
        and enclosing_func.type.ret_type.type.fullname() == "typing.Coroutine"
        and len(enclosing_func.type.ret_type.args) == 3
    ):
        yield_type, send_type, _ = decode_agen_types_from_return_type(
            ctx, enclosing_func.type.ret_type.args[2]
        )
        return yield_type, send_type

    return (
        AnyType(TypeOfAny.implementation_artifact),
        AnyType(TypeOfAny.implementation_artifact),
    )


def yield_callback(ctx: FunctionContext) -> Type:
    """Provide a more specific argument and return type for yield_()
    inside an @async_generator.
    """
    if len(ctx.arg_types) == 0:
        arg_type = NoneTyp(ctx.context.line, ctx.context.column)  # type: Type
    elif ctx.arg_types and len(ctx.arg_types[0]) == 1:
        arg_type = ctx.arg_types[0][0]
    else:
        return ctx.default_return_type

    private_api = cast(TypeChecker, ctx.api)
    yield_type, send_type = decode_enclosing_agen_types(ctx)
    if yield_type is not None and send_type is not None:
        private_api.check_subtype(
            subtype=arg_type,
            supertype=yield_type,
            context=ctx.context,
            subtype_label="yield_ argument",
            supertype_label="declared YieldType",
        )
        return ctx.api.named_generic_type("typing.Awaitable", [send_type])

    return ctx.default_return_type


def yield_from_callback(ctx: FunctionContext) -> Type:
    """Provide a better typecheck for yield_from_()."""
    if ctx.arg_types and len(ctx.arg_types[0]) == 1:
        arg_type = ctx.arg_types[0][0]
    else:
        return ctx.default_return_type

    private_api = cast(TypeChecker, ctx.api)
    our_yield_type, our_send_type = decode_enclosing_agen_types(ctx)
    if our_yield_type is None or our_send_type is None:
        return ctx.default_return_type

    if (
        isinstance(arg_type, Instance)
        and arg_type.type.fullname()
        in (
            "trio_typing.CompatAsyncGenerator",
            "trio_typing.AsyncGenerator",
            "typing.AsyncGenerator",
        )
        and len(arg_type.args) >= 2
    ):
        their_yield_type, their_send_type = arg_type.args[:2]
        private_api.check_subtype(
            subtype=their_yield_type,
            supertype=our_yield_type,
            context=ctx.context,
            subtype_label="yield_from_ argument YieldType",
            supertype_label="local declared YieldType",
        )
        private_api.check_subtype(
            subtype=our_send_type,
            supertype=their_send_type,
            context=ctx.context,
            subtype_label="local declared SendType",
            supertype_label="yield_from_ argument SendType",
        )
    elif isinstance(arg_type, Instance):
        private_api.check_subtype(
            subtype=arg_type,
            supertype=ctx.api.named_generic_type(
                "typing.AsyncIterable", [our_yield_type]
            ),
            context=ctx.context,
            subtype_label="yield_from_ argument type",
            supertype_label="expected iterable type",
        )

    return ctx.default_return_type


def started_callback(ctx: MethodContext) -> Type:
    """Raise an error if task_status.started() is called without an argument
    and the TaskStatus is not declared to accept a result of type None.
    """
    if (
        (not ctx.arg_types or not ctx.arg_types[0])
        and isinstance(ctx.type, Instance)
        and ctx.type.args
        and not isinstance(ctx.type.args[0], NoneTyp)
    ):
        ctx.api.fail(
            "TaskStatus.started() requires an argument for types other than "
            "TaskStatus[None]",
            ctx.context,
        )
    return ctx.default_return_type


def takes_callable_and_args_callback(ctx: FunctionContext) -> Type:
    """Automate the boilerplate for writing functions that accept
    arbitrary positional arguments of the same type accepted by
    a callable.

    For example, this supports writing::

        @trio_typing.takes_callable_and_args
        def start_soon(
            self,
            async_fn: Callable[[VarArg()], None],
            *args: Any,
        ) -> None: ...

    instead of::

        T1 = TypeVar("T1")
        T2 = TypeVar("T2")
        T3 = TypeVar("T3")
        T4 = TypeVar("T4")

        @overload
        def start_soon(
            self,
            async_fn: Callable[[], None],
        ) -> None: ...

        @overload
        def start_soon(
            self,
            async_fn: Callable[[T1], None],
            __arg1: T1,
        ) -> None: ...

        @overload
        def start_soon(
            self,
            async_fn: Callable[[T1, T2], None],
            __arg1: T1,
            __arg2: T2
        ) -> None: ...

        # etc

    """
    try:
        if (
            not ctx.arg_types
            or len(ctx.arg_types[0]) != 1
            or not isinstance(ctx.arg_types[0][0], CallableType)
            or not isinstance(ctx.default_return_type, CallableType)
        ):
            raise ValueError("must be used as a decorator")

        fn_type = ctx.arg_types[0][0]  # type: CallableType
        callable_idx = -1  # index in function arguments of the callable
        callable_args_idx = -1  # index in callable arguments of the StarArgs
        callable_ty = None  # type: Optional[CallableType]
        args_idx = -1  # index in function arguments of the StarArgs

        for idx, (kind, ty) in enumerate(zip(fn_type.arg_kinds, fn_type.arg_types)):
            if isinstance(ty, AnyType) and kind == ARG_STAR:
                assert args_idx == -1
                args_idx = idx
            elif isinstance(ty, (UnionType, CallableType)) and kind == ARG_POS:
                # turn Union[Callable[..., T], Callable[[VarArg()], T]]
                # into Callable[[VarArg()], T]
                # (the union makes it not fail when the plugin is not being used)
                if isinstance(ty, UnionType):
                    for arm in ty.items:
                        if (
                            isinstance(arm, CallableType)
                            and not arm.is_ellipsis_args
                            and any(kind_ == ARG_STAR for kind_ in arm.arg_kinds)
                        ):
                            ty = arm
                            break
                    else:
                        continue

                for idx_, (kind_, ty_) in enumerate(zip(ty.arg_kinds, ty.arg_types)):
                    if isinstance(ty_, AnyType) and kind_ == ARG_STAR:
                        if callable_idx != -1:
                            raise ValueError(
                                "decorated function may only take one callable "
                                "that has an argument of type VarArg()"
                            )
                        callable_idx = idx
                        callable_args_idx = idx_
                        callable_ty = ty

        if args_idx == -1:
            raise ValueError("decorated function must take *args: Any")
        if callable_idx == -1 or callable_ty is None:
            raise ValueError(
                "decorated function must take a callable that has a "
                "argument of type mypy_extensions.VarArg()"
            )

        expanded_fns = []  # type: List[CallableType]
        type_var_defs = []  # type: List[TypeVarDef]
        type_var_types = []  # type: List[Type]
        for arg_idx in range(1, 7):  # provides overloads for 0 through 5 arguments
            arg_types = list(fn_type.arg_types)
            arg_types[callable_idx] = callable_ty.copy_modified(
                arg_types=(
                    callable_ty.arg_types[:callable_args_idx]
                    + type_var_types
                    + callable_ty.arg_types[callable_args_idx + 1 :]
                ),
                arg_kinds=(
                    callable_ty.arg_kinds[:callable_args_idx]
                    + ([ARG_POS] * len(type_var_types))
                    + callable_ty.arg_kinds[callable_args_idx + 1 :]
                ),
                arg_names=(
                    callable_ty.arg_names[:callable_args_idx]
                    + ([None] * len(type_var_types))
                    + callable_ty.arg_names[callable_args_idx + 1 :]
                ),
                variables=(callable_ty.variables + type_var_defs),
            )
            expanded_fns.append(
                fn_type.copy_modified(
                    arg_types=(
                        arg_types[:args_idx]
                        + type_var_types
                        + arg_types[args_idx + 1 :]
                    ),
                    arg_kinds=(
                        fn_type.arg_kinds[:args_idx]
                        + ([ARG_POS] * len(type_var_types))
                        + fn_type.arg_kinds[args_idx + 1 :]
                    ),
                    arg_names=(
                        fn_type.arg_names[:args_idx]
                        + ([None] * len(type_var_types))
                        + fn_type.arg_names[args_idx + 1 :]
                    ),
                    variables=(fn_type.variables + type_var_defs),
                )
            )
            type_var_defs.append(
                TypeVarDef(
                    "__T{}".format(arg_idx),
                    "__T{}".format(arg_idx),
                    -len(fn_type.variables) - arg_idx - 1,
                    [],
                    ctx.api.named_generic_type("builtins.object", []),
                )
            )
            type_var_types.append(
                TypeVarType(type_var_defs[-1], ctx.context.line, ctx.context.column)
            )
        return Overloaded(expanded_fns)

    except ValueError as ex:
        ctx.api.fail(
            "invalid use of @takes_callable_and_args: {}".format(ex), ctx.context
        )
        return ctx.default_return_type


def plugin(version: str) -> typing_Type[Plugin]:
    return TrioPlugin



import trio
from typing import Any, Callable, Optional, TypeVar, Union
from trio_typing import takes_callable_and_args
from mypy_extensions import VarArg

T = TypeVar("T")

def current_default_thread_limiter() -> trio.CapacityLimiter: ...
@takes_callable_and_args
async def run_sync(
    sync_fn: Union[Callable[..., T], Callable[[VarArg()], T]],
    *args: Any,
    cancellable: bool = False,
    limiter: Optional[trio.CapacityLimiter] = None,
) -> T: ...



# Adapts mypy.test.data pytest plugin for use outside the mypy tree

import os
import sys
import types

data_prefix = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "trio_typing/_tests/test-data"
)


class ConfigModule(types.ModuleType):
    def __init__(self) -> None:
        self.test_data_prefix = data_prefix
        self.PREFIX = os.path.dirname(os.path.realpath(__file__))
        self.test_temp_dir = "tmp"


sys.modules["mypy.test.config"] = ConfigModule()

from mypy.test.data import *



import attr
import trio
from typing import (
    Any,
    AsyncContextManager,
    Awaitable,
    Callable,
    ContextManager,
    Optional,
    Tuple,
    TypeVar,
    Union,
)

F = TypeVar("F", bound=Callable[..., Any])

async def wait_all_tasks_blocked(
    cushion: float = ..., tiebreaker: int = ...
) -> None: ...
def trio_test(fn: F) -> F: ...

class MockClock(trio.abc.Clock):
    rate: float
    autojump_threshold: float
    def __init__(self, rate: float = ..., autojump_threshold: float = ...): ...
    def start_clock(self) -> None: ...
    def current_time(self) -> float: ...
    def deadline_to_sleep_time(self, deadline: float) -> float: ...
    def jump(self, seconds: float) -> None: ...

def assert_checkpoints() -> ContextManager[None]: ...
def assert_no_checkpoints() -> ContextManager[None]: ...

class Sequencer:
    def __call__(self, position: int) -> AsyncContextManager[None]: ...

_StreamMaker = Callable[
    [], Awaitable[Tuple[trio.abc.SendStream, trio.abc.ReceiveStream]]
]

async def check_one_way_stream(
    stream_maker: _StreamMaker, clogged_stream_maker: Optional[_StreamMaker]
) -> None: ...
async def check_two_way_stream(
    stream_maker: _StreamMaker, clogged_stream_maker: Optional[_StreamMaker]
) -> None: ...
async def check_half_closeable_stream(
    stream_maker: _StreamMaker, clogged_stream_maker: Optional[_StreamMaker]
) -> None: ...

MemoryStreamHook = Optional[Callable[[], Awaitable[None]]]
MemoryStreamSyncHook = Optional[Callable[[], None]]
@attr.s(auto_attribs=True)
class MemorySendStream(trio.abc.SendStream):
    send_all_hook: MemoryStreamHook = ...
    wait_send_all_might_not_block_hook: MemoryStreamHook = ...
    close_hook: MemoryStreamSyncHook = ...
    async def send_all(self, data: Union[bytes, bytearray, memoryview]) -> None: ...
    async def wait_send_all_might_not_block(self) -> None: ...
    async def aclose(self) -> None: ...
    def close(self) -> None: ...
    async def get_data(self, max_bytes: Optional[int] = ...) -> bytes: ...
    def get_data_nowait(self, max_bytes: Optional[int] = ...) -> bytes: ...

@attr.s(auto_attribs=True)
class MemoryReceiveStream(trio.abc.ReceiveStream):
    receive_some_hook: MemoryStreamHook = ...
    close_hook: MemoryStreamSyncHook = ...
    async def receive_some(self, max_bytes: Optional[int] = ...) -> bytes: ...
    async def aclose(self) -> None: ...
    def close(self) -> None: ...
    def put_data(self, data: bytes) -> None: ...
    def put_eof(self) -> None: ...

def memory_stream_pump(
    memory_send_stream: MemorySendStream,
    memory_receive_stream: MemoryReceiveStream,
    *,
    max_bytes: Optional[int] = ...,
) -> bool: ...
def memory_stream_one_way_pair() -> Tuple[MemorySendStream, MemoryReceiveStream]: ...
def memory_stream_pair() -> Tuple[trio.StapledStream, trio.StapledStream]: ...
def lockstep_stream_one_way_pair() -> Tuple[MemorySendStream, MemoryReceiveStream]: ...
def lockstep_stream_pair() -> Tuple[trio.StapledStream, trio.StapledStream]: ...
async def open_stream_to_socket_listener(
    socket_listener: trio.SocketListener
) -> trio.SocketStream: ...
